
---------------------------------------------
-- CREATE TABLE FCM_ESTIMATE_RULE
---------------------------------------------
CREATE TABLE FCM_ESTIMATE_RULE (
  ESTIMATE_ID	BIGINT	NOT NULL,
  RULE_ID	INTEGER	NOT NULL,
  BASE_RULE_ID	INTEGER	NOT NULL,
  RULE_VERSION_NBR	INTEGER	NOT NULL,
  RULE_TYPE_NAME	VARCHAR(50)	NOT NULL,
  ALERT_TYPE	VARCHAR(1),
  RULE_NAME	VARCHAR(150)	NOT NULL,
  RULE_DESC	VARCHAR(765),
  RULE_STATE	VARCHAR(20),
  RULE_CODE	CLOB(1G)	 INLINE LENGTH 276	LOGGED	COMPACT,
  RULE_ORDER_TYPE	INTEGER	NOT NULL	DEFAULT ,
  RULE_ORDER_PRIORITY	INTEGER,
  EST_RIX	INTEGER,
  MULTI_ORG_NODE_KEY	VARCHAR(6),
  MULTI_ORG_NAME	VARCHAR(16),
  ACCT_VALUES	VARCHAR(4000),
  ACTI_VALUES	VARCHAR(4000),
  CHAN_VALUES	VARCHAR(4000),
  AUTH_VALUES	VARCHAR(4000),
  CUST_VALUES	VARCHAR(4000),
  CREATE_USER	VARCHAR(60)	NOT NULL,
  CREATE_TIMESTAMP	TIMESTAMP	NOT NULL	DEFAULT ,
  LSTUPDT_USER	VARCHAR(60),
  LSTUPDT_TIMESTAMP	TIMESTAMP,
  LSTUPDT_NBR	SMALLINT	NOT NULL	DEFAULT 
  ) 
  IN TS_CORE
  INDEX IN IX_CORE;

---------------------------------------------
-- CREATE INDEXES 
---------------------------------------------
CREATE UNIQUE INDEX PKX_ESTIMATE_RULE
  ON FCM_ESTIMATE_RULE
    ( ESTIMATE_ID ASC, RULE_ID ASC )
  CLUSTER
  ALLOW REVERSE SCANS
  COMPRESS YES
  INCLUDE NULL KEYS;

CREATE INDEX FKX1_ESTIMATE_RULE
  ON FCM_ESTIMATE_RULE
    ( ESTIMATE_ID ASC )
  ALLOW REVERSE SCANS
  COMPRESS YES
  INCLUDE NULL KEYS;
  
---------------------------------------------
-- CREATE PK Constraint PKX_ESTIMATE_RULE
---------------------------------------------
ALTER TABLE FCM_ESTIMATE_RULE
  ADD CONSTRAINT PKX_ESTIMATE_RULE PRIMARY KEY
    (ESTIMATE_ID, RULE_ID)
    ENFORCED;

---------------------------------------------
-- CREATE FK Constraint FK_EST_RULE_EST
---------------------------------------------
ALTER TABLE FCM_ESTIMATE_RULE
  ADD CONSTRAINT FK_EST_RULE_EST FOREIGN KEY
    (ESTIMATE_ID)
  REFERENCES FCM_ESTIMATE
    (ESTIMATE_ID)
    ON DELETE CASCADE
    ON UPDATE RESTRICT
    ENFORCED
    ENABLE QUERY OPTIMIZATION;

---------------------------------------------
-- Authorizations
---------------------------------------------
GRANT INSERT,SELECT,UPDATE,DELETE on TABLE FCM_ESTIMATE_RULE to ROLE SAS_SOR_APP;
GRANT INSERT,SELECT,UPDATE,DELETE on TABLE FCM_ESTIMATE_RULE to ROLE SAS_SOR_BATCH;

---------------------------------------------------------------------------------
--
-- Insert version release 4.3.0.6 (4.3 M0 HF6)
--
---------------------------------------------------------------------------------
merge into FCM_VERSION p
USING TABLE(values('SOR Base DDL', '4.3.0.6'))
                     ac(VERSION_NAME , VERSION_NUM)
                  ON (p.VERSION_NAME = ac.VERSION_NAME
                      AND p.VERSION_NUM=ac.VERSION_NUM)
when not matched then
insert (
    VERSION_NAME , VERSION_NUM , VERSION_DATE , VERSION_DESC, VERSION_COMPONENT)
VALUES (
        'SOR Base DDL', '4.3.0.6', current_date, '4.3 M0 HF6 DDL', 'SOR DDL')
;

commit;


