
---------------------------------------------
-- CREATE TABLE FCM_ESTIMATE_RULE
---------------------------------------------
CREATE TABLE FCM_ESTIMATE_RULE
(
  ESTIMATE_ID          NUMBER(19)               NOT NULL,
  RULE_ID              NUMBER(10)               NOT NULL,
  BASE_RULE_ID         NUMBER(10)               NOT NULL,
  RULE_VERSION_NBR     NUMBER(10)               NOT NULL,
  RULE_TYPE_NAME       VARCHAR2(50 BYTE)        NOT NULL,
  ALERT_TYPE           VARCHAR2(1 BYTE),
  RULE_NAME            VARCHAR2(150 BYTE)       NOT NULL,
  RULE_DESC            VARCHAR2(765 BYTE),
  RULE_STATE           VARCHAR2(20 BYTE),
  RULE_CODE            CLOB,
  RULE_ORDER_TYPE      NUMBER(10)               DEFAULT 0                     NOT NULL,
  RULE_ORDER_PRIORITY  NUMBER(10),
  EST_RIX              NUMBER(10),
  MULTI_ORG_NODE_KEY   VARCHAR2(6 BYTE),
  MULTI_ORG_NAME       VARCHAR2(16 BYTE),
  ACCT_VALUES          VARCHAR2(4000 BYTE),
  ACTI_VALUES          VARCHAR2(4000 BYTE),
  CHAN_VALUES          VARCHAR2(4000 BYTE),
  AUTH_VALUES          VARCHAR2(4000 BYTE),
  CUST_VALUES          VARCHAR2(4000 BYTE),
  CREATE_USER          VARCHAR2(60 BYTE)        NOT NULL,
  CREATE_TIMESTAMP     TIMESTAMP(6)             DEFAULT CURRENT_TIMESTAMP     NOT NULL,
  LSTUPDT_USER         VARCHAR2(60 BYTE),
  LSTUPDT_TIMESTAMP    TIMESTAMP(6),
  LSTUPDT_NBR          NUMBER(5)                DEFAULT 0                     NOT NULL
)
LOB (RULE_CODE) STORE AS SECUREFILE (
  TABLESPACE  TS_CORE
  ENABLE      STORAGE IN ROW
  CHUNK       32768
  NOCACHE
  LOGGING)
TABLESPACE TS_CORE;

---------------------------------------------
-- CREATE INDEXES 
---------------------------------------------
CREATE INDEX FKX1_ESTIMATE_RULE ON FCM_ESTIMATE_RULE
(ESTIMATE_ID)
LOGGING
TABLESPACE IX_CORE;

CREATE UNIQUE INDEX PKX_ESTIMATE_RULE ON FCM_ESTIMATE_RULE
(ESTIMATE_ID, RULE_ID)
LOGGING
TABLESPACE IX_CORE;

---------------------------------------------
-- CREATE PK Constraint PKX_ESTIMATE_RULE
---------------------------------------------
ALTER TABLE FCM_ESTIMATE_RULE ADD (
  CONSTRAINT PKX_ESTIMATE_RULE
  PRIMARY KEY
  (ESTIMATE_ID, RULE_ID)
  USING INDEX PKX_ESTIMATE_RULE
  ENABLE VALIDATE);

---------------------------------------------
-- CREATE FK Constraint FK_EST_RULE_EST
--------------------------------------------
ALTER TABLE FCM_ESTIMATE_RULE ADD (
  CONSTRAINT FK_EST_RULE_EST 
  FOREIGN KEY (ESTIMATE_ID) 
  REFERENCES FCM_ESTIMATE (ESTIMATE_ID)
  ON DELETE CASCADE
  ENABLE VALIDATE);



---------------------------------------------
-- Authorizations
---------------------------------------------
GRANT INSERT,SELECT,UPDATE,DELETE on FCM_ESTIMATE_RULE to SAS_SOR_APP;
GRANT INSERT,SELECT,UPDATE,DELETE on FCM_ESTIMATE_RULE to SAS_SOR_BATCH;


	
---------------------------------------------------------------------------------
--
-- Insert version release 4.3.0.6 (4.3 M0 HF6)
--
---------------------------------------------------------------------------------
merge into FCM_VERSION p
USING (select 'Base DDL' as VERSION_NAME,'4.3.0.6' as VERSION_NUM from dual) ac
                  ON (p.VERSION_NAME = ac.VERSION_NAME AND p.VERSION_NUM=ac.VERSION_NUM)
when not matched then
insert (
    VERSION_NAME , VERSION_NUM , VERSION_DATE , VERSION_DESC, VERSION_COMPONENT) 
VALUES (
 	'Base DDL', '4.3.0.6', current_date, '4.3 M0 HF6 DDL', 'DDL')
;
commit;
