%macro apply_replacement_methodology(
    in = ,
    out = ,
    replacement_win_days = 7,
    replacement_win_trans = 3
    );

options sortvalidate;
proc sort tagsort data = &in;
    by &entity rqo_proc_utc_datetime rqo_tran_date rqo_tran_time;
run;

data &out;

    if _n_ = 1 then do;
        length
            tag3_cnt 8
            &entity $20;
        drop tag3_cnt;
        declare hash repl_ht(hashexp: 16);
        rc = repl_ht.defineKey("&entity");
        rc = repl_ht.defineData('tag3_cnt');
        rc = repl_ht.defineDone();
        call missing(&entity, tag3_cnt);
    end;    

    set &in;
    
    afd_frd_tag_orig = afd_frd_tag;

    drop rqo_tran_datetime;
    rqo_tran_datetime = sum(86400*rqo_tran_date, rqo_tran_time);

    if afd_frd_tag='0' then do;
    end;    
    else if afd_frd_tag='9' then do;
        delete;
    end;
    else if afd_frd_tag in ('2','1','3') then do;
        if missing(AFD_DOFF) or missing(AFD_DOFF_TIME) then do;
            putlog "ERROR: Missing DOFF " &entity= afd_frd_tag= AFD_DOFF= AFD_DOFF_TIME=;
            delete;
        end;
        else if missing(AFD_BLOCK_DATE) or missing(AFD_BLOCK_TIME) then do;
            putlog "ERROR: Missing Block dates " &entity= afd_frd_tag= AFD_BLOCK_DATE= AFD_BLOCK_TIME=;
            delete;
        end;
        else do;
            drop doff blckdt;
            
            doff=sum(86400*AFD_DOFF,AFD_DOFF_TIME);
            blckdt=sum(86400*AFD_BLOCK_DATE,AFD_BLOCK_TIME);

            if afd_frd_tag='3' then do;
                drop rc_: rc;
                rc_repl=repl_ht.find();
                if rc_repl=0 then do;
                    tag3_cnt=tag3_cnt+1;
                    rc_repl2=repl_ht.replace();
                end;
                else do;
                    tag3_cnt=1;
                    rc_repl2=repl_ht.add();
                end;

                /* Flip tag 3 to tag 1 for the transactions in the replacement window*/
                if 0 < rqo_tran_datetime - blckdt < 86400*&replacement_win_days
                    and 0< tag3_cnt <=&replacement_win_trans then
                    afd_frd_tag='1';
                end;

            end;
        end;
     else do;
         putlog "ERROR: Unrecognized fraud tag " &entity= afd_frd_tag=;
         delete;
     end;
 
run;

%mend;    
    
    
