%macro basic_stats(driver_ds=stat_current, interior_cellwidth=1.5in, title=Basic Statistics, dollar_fmt=dollar15.);

%numobs(data=&driver_ds);

%delit(ds=temp);
%delit(ds=stats);

data _null_;
do k=1 to 2;
do j=1 to &numobs;
   set work.&driver_ds point=j;


 if k=1 and j=1 then do;
 call execute("Data Stats;");
 call execute("length label $250;");
 call execute("Set ");
 end;

 if k=1 then call execute("&qtrlib.."||strip(memname)||"(in=ds_"||strip(j)||")");
 if k=2 and j=1 then call execute(";");

 if k=2 then do;
  call execute("if ds_"||strip(j)||" then label='"||strip(label)||"';");
 end;

 if k=2 and j=&numobs then call execute("run;");
 end;
end;
 stop;
run;

proc transpose data=stats out=temp;
 id label;
 idlabel label;
 run;

 data temp;
  set temp;
  holder=1;
  run;

%let totobs=%left(%eval(&numobs+2));
%put totobs: &totobs;

 proc sql noprint;
  select name, varnum, label
  into :Name1-:Name&totobs,
       :Varnum1-:Varnum&totobs,
	   :Label1-:Label&totobs
  from dictionary.columns
  where libname='WORK' and memname='TEMP';

ods proclabel "Descriptive Statistics";

  proc report data=temp split=' to ' style(header)=[verticalalign=bottom] contents="";
   title "&title";
   columns _label_  %do j=3 %to &totobs; %str(&&name&j) %end;  dummy holder;
   define _label_ /"Metric" display ;
 
   %do j=3 %to &totobs;
   define &&name&j/display style(column)=[cellwidth=&interior_cellwidth];
   %end;

   define dummy/computed noprint;
   define holder/order noprint;
   break before holder/page contents='';
   compute dummy;
      if _c1_ in('Evaluation Begin Date','Evaluation End Date') then do;
	        %do j=2 %to %eval(&numobs+1);
              call define("_c&j._",'format','mmddyy10.');
			%end;
	  end;

	  else if _c1_ in('Total Card Count', 'Total Transaction Count', 'Total Case Count', 'Total Transaction Count', 'Total Approved Trans Count') then do;
	        %do j=2 %to %eval(&numobs+1);
			  call define("_c&j._",'format','comma14.');
			%end;
	  end;

	  else if _c1_ in('Total Approved Amount') then do;
	        %do j=2 %to %eval(&numobs+1);
			  call define("_c&j._",'format',"&dollar_fmt");
			%end;
	  end;


	  else if _c1_ in('Trans Approval Rate','Monthly Case Fraud Rate','Trans Fraud Rate','Value Fraud Rate') then do;
	        %do j=2 %to %eval(&numobs+1);
			  call define("_c&j._",'format','percent10.2');
			%end;
	  end;

	  else if _c1_ in('Avg Trans Cnt / Cards','Avg Trans Cnt per Case')  then do;
	       %do j=2 %to %eval(&numobs+1);
		     call define("_c&j._",'format','10.2');
		   %end;
	  end;
	endcomp;
	run;

%delit(ds=temp);
%delit(ds=stats);

%mend basic_stats;
