%macro dateprep(TYPE= );

%local report_date_raw;
%let report_date_raw = &report_date;

%if &report_date_raw eq  %then %do;
    data _null_;
        x=today();
        %if &type=DAILY %then call symputx('report_date',x-1);;
        %if &type=MONTHLY %then call symputx('report_date',x);;
    run;
    %let report_date9=%sysfunc(putn(&report_date,date9.));
    %put Note:  Report Date Was Not Entered;
    %put Derived Report Date Is: %sysfunc(putn(&report_date,worddate.));
    %put Today is:  %sysfunc(today(),worddate.);
    %put Type of Report is: %lowcase(&type);
    %put Daily should be yesterday and monthly should as of today;
%end;
%else %do;  **case where report date is entered;
    data _null_;
        x="&report_date_raw"d;
        %if &type=DAILY %then call symputx('report_date',x);;
        %if &type=MONTHLY %then call symputx('report_date',x+1);;
    run;
    %let report_date9=%sysfunc(putn(&report_date,date9.));
    %put Note:  Report Date Was Entered;
    %put Entered Report Date Is: %sysfunc(putn("&report_date_raw"d, worddate.));
    %put Derived Report Date Is: %sysfunc(putn(&report_date,worddate.));
    %put Today is:  %sysfunc(today(),worddate.);
    %put Type of Report is: %lowcase(&type);
    %put Daily should be yesterday and Monthly should include Report Date Plus One;
%end;


%global development_begin_date development_end_date;
%let development_begin_date=.;
%let development_end_date =.; 

%if %sysfunc(exist(devdata.development_dates,data)) = 1 %then
%do; 
   data _null_;
    set devdata.development_dates;
    call symputx('development_begin_date', put(development_begin_date, best.), 'G');
    call symputx('development_end_date', put(development_end_date, best.), 'G');
   run;
%end;


data _null_;
    rundate=today();
    report_date=&report_date;
    sampling_dig = ceil(1000 * &rh_sampling_rate);
    wts_var = ceil(1000 / sampling_dig);

    call symputx('rundate',rundate,'G');

    %if &TYPE eq DAILY %then %do;
        previous_day=intnx('day',report_date,-1,'same');
        same_day_last_week=intnx('week',report_date,-1,'same');

        call symputx('previous_day', put(previous_day, best.),'G');
        call symputx('same_day_last_week', put(same_day_last_week, best.), 'G');
        call symputx('extract_wherecond',
            catx(" ", "rqo_tran_date in (", put(report_date, best.), ",", put(previous_day, best.),
            ", ", put(same_day_last_week, best.), ")" ),
            'G');
       
            call symputx('extract_wherecond_rh',
                catx(" ", "rqo_tran_date in (", put(report_date, best.), ",", put(previous_day, best.),
                                             ", ", put(same_day_last_week, best.), ")" ),
                'G');
      
        call symputx("wts_var", "1", "G");
    %end;
    %else %if &TYPE eq MONTHLY %then %do;
        current_month_start=intnx('month',report_date,-1,'beginning');
        current_month_end=intnx('month',report_date,-1,'end');
        previous_month_start=intnx('month',report_date,-2,'beginning');
        previous_month_end=intnx('month',report_date,-2,'end');

        call symputx('current_month_start',current_month_start,'G');
        call symputx('current_month_end',current_month_end,'G');
        call symputx('previous_month_start',previous_month_start,'G');
        call symputx('previous_month_end',previous_month_end,'G');

        call symputx('extract_wherecond',
            catx(" ", put(previous_month_start, best.), "<= rqo_tran_date <=", put(current_month_end, best.),
                      "and &sampling_dig < ", put(sampling_dig, best.) ),
            'G');
        
            call symputx('extract_wherecond_rh',
                catx(" ", put(previous_month_start, best.), "<= rqo_tran_date <=", put(current_month_end, best.),
                          "and",
                          "&sampling_dig < ", put(sampling_dig, best.) ),
                'G');
       
        call symputx("wts_var", put(wts_var, best.), "G");
    %end;
    %else %if &TYPE eq DEV_DAILY %then %do;
        call symputx('extract_wherecond',
            catx(" ", "&development_begin_date", "<= rqo_tran_date <=", "&development_end_date"),
            'G');
        call symputx("wts_var", "_weight_", "G");
    %end;
    %else %if &TYPE eq DEV_MONTHLY %then %do;
        call symputx('extract_wherecond',
                     catx(" ", "&development_begin_date", "<= rqo_tran_date <=", "&development_end_date"),
                     'G');
        call symputx("wts_var", "_weight_", "G");
    %end;
run;

%put rundate: %sysfunc(putn(&rundate,worddate.));
%put development_begin_date: %sysfunc(putn(&development_begin_date,worddate.));
%put development_end_date: %sysfunc(putn(&development_end_date,worddate.));

%if &TYPE eq DAILY %then %do;
    %put report_date: %sysfunc(putn(&report_date,worddate.));
    %put previous_day: %sysfunc(putn(&previous_day,worddate.));
    %put same_day_last_week: %sysfunc(putn(&same_day_last_week,worddate.));
%end;

%if &TYPE eq MONTHLY %then %do;
    %put report_date: %sysfunc(putn(&report_date,worddate.));
    %put current_month_start: %sysfunc(putn(&current_month_start,worddate.));
    %put current_month_end: %sysfunc(putn(&current_month_end,worddate.));
    %put previous_month_start: %sysfunc(putn(&previous_month_start,worddate.));
    %put previous_month_end: %sysfunc(putn(&previous_month_end,worddate.));
%end;

%put extract_wherecond: &extract_wherecond;


%mend dateprep;
