 %macro prep_driver_ds_interpolate_ds(qtrlib=);

*---------------------------------------------------* 
| Setup Parameters                                  | 
*---------------------------------------------------*; 
%global devb deve;
%let qtrlib=%upcase(&qtrlib);
%let num_historical_periods = 2 ; 

%let Interpolate1=CFPR; 
%let start1=0; 
%let incr1=5; 
%let stop1=200; 

%let Interpolate2=outsort_1000000; 
%let start2=0; 
%let incr2=1000;
%let stop2=20000;

%let depvars=cutoff CDR VDR LCR &Interpolate1 &Interpolate2 outsort OSF cutoff numFP dolFP detFrdDol totFrdDol detFrdAcct totLgtDol 
    srate ; 

*---------------------------------------------------* 
| Create Custom Format to be Used for Labels        | 
*---------------------------------------------------*; 

    
   /*** version 9.3 - create a function and format from function ***/
   /*** this process can be quirky - so execute only once***/

%if %sysevalf(&sysver ge 9.3) %then %do;
	proc sql;
	 create table check_format as
	 select objname
	 from dictionary.formats
	 where libname='WORK' and objname='CUSTDATE';

	%numobs(check_format)

	%if &numobs eq 0 %then %do;
	    
	    %delit(ds=work.functions);
		
		proc fcmp outlib=work.functions.date;
		  function custdate(date) $ 8;
		   formatted_date=put(date,monname3.)||"-"||put(date,year4.);
		  return(formatted_date);
		 endsub;

		options cmplib=work.functions;

		proc format;
		 value custdate (default=8)  other=[custdate()];
		run;

	%end;
%end;

%else %do;     ***version 9.2 doe not get '-' in label ***;
		proc format;
		 value custdate (default=8)  other=[monyy7.];
		run;

%end;

*---------------------------------------------------* 
| Prepare Driver Data Sets                          | 
*---------------------------------------------------*; 

%let vlist=EVAL STAT; 

%let loopnum=1; 
%do %until(%scan(&vlist,&loopnum,%str( )) eq %str()); 
 %let loopvar=%trim(%scan(&vlist,&loopnum,%str( ))); 

proc sql number; 
 create table &loopvar(drop=node1-node3) as 
 select memname, scan(memname,1,'_') as Node1 length=11,  strip(scan(memname,2,'_')) as node2, strip(scan(memname,3,'_')) as node3, 
         case 
              when calculated node2='DEV' then calculated node2 
              else calculated node1 
         end as Report_Type length=4, 
         case 
              when calculated node1 in('EVAL','STAT') and calculated node2 ne 'DEV' then input(calculated node2,date9.) 
              else . 
         end as Date1 format=mmddyy10., 
         case 
              when not(missing(calculated date1)) then input(calculated node3,date9.) 
              else . 
         end as date2 format=mmddyy10., 
         case 
              when not(missing(calculated date1)) and not(missing(calculated date2)) 
              then cat("Prod ",strip(put(calculated date1,custdate.))," to ",strip(put(calculated date2,custdate.))) 
              else "Development" 
         end as label length=50,
         nobs as number_observations 
 from dictionary.tables 
 where libname="&qtrlib" and memname ne 'DEVELOPMENT_DATES' 
 having node1="&LOOPVAR" 
 order by node1, date2 desc; 

*-------------------------------------------------------------------------------------* 
| Only Data Sets with the Correct Number of Observations are considered for reporting |
*-------------------------------------------------------------------------------------*; 

 %if &loopvar=EVAL %then %do;
  data &loopvar;
   set &loopvar;
   where number_observations=1001;
   run;
 %end;
 %else %if &loopvar=STAT %then %do;
  data &loopvar;
   set &loopvar;
   where number_observations=1;
   run;
 %end;

*---------------------------------------------------* 
| Find largest end date less than report date       | 
*---------------------------------------------------*; 

proc sql noprint;
 select max(date2) as max_date, intnx('year',calculated max_date,-1,'same')  
 into :max_date, :one_year_prior
 from &loopvar
 where date2<=&eval_win_end_date; ***find the largest end date;

%put ; %put ;
%put Largest End Date:  %sysfunc(putn(&max_date,mmddyy10.));
%put Same Quarter Last Year: %sysfunc(putn(&one_year_prior,mmddyy10.));  
%put ; %put ;


*-------------------------------------------------------------------------------*
| Subset the list of all possible Eval/Stat Data Sets to current reporting list |
*-------------------------------------------------------------------------------*; 

data &loopvar._current; 
 row_number=_n_; 
 set &loopvar(where=(date2<=&eval_win_end_date));

if Report_type='DEV' then do;
   date1temp=put(input(scan(memname,3,"_"),date9.),custdate.);
   date2temp=put(input(scan(memname,4,"_"),date9.),custdate.);
   call symputx('devb',date1temp,'G');
   call symputx('deve',date2temp,'G');
   label="Dev"||" "||strip(date1temp)||" to "||strip(date2temp);
end;

if (report_type="&LOOPVAR" and row_number<=&num_historical_periods) OR (date2=&one_year_prior) OR (Report_type="DEV");

drop date1temp date2temp;
run; 

*------------------------------------------------------------------* 
| Add Outsort Rate Times 1000000 to All Eval Data Sets             |                                                  
*------------------------------------------------------------------*; 

%if &loopvar eq EVAL %then %do; 

    *------------------------------------------------------------------------------------------------------------------------------* 
    | The following code checks to see if outsort rate already exists - it adds a flag 'outsort_exist_1000000' to the eval dataset | 
    *------------------------------------------------------------------------------------------------------------------------------*; 
    data _null_; 
       set work.eval nobs=numobs; 
       call symputx("memname_"||strip(_n_),memname); 
       if _n_=1 then call symputx("totrecs",numobs); 
    run; 

    %do j=1 %to &totrecs; 
      %let varexist_&j=%varexist(ds=&qtrlib..&&memname_&j,var=outsort_1000000); 
    %end; 

    data work.eval; 
     set work.eval; 
     %do j=1 %to &totrecs; 
       %if &j>1 %then %str(else); if memname="%trim(&&memname_&j)" then outsort_exist_1000000=&&varexist_&j; 
     %end; 
    run; 
    *------------------------------------------------------------------------------------------------------------------------------* 
    | The following code checks to see if outsort rate already exists - it checks the the flag 'outsort_exist_1000000'             | 
    *------------------------------------------------------------------------------------------------------------------------------*; 

data _null_; 
 set work.eval; 

 if outsort_exist_1000000=0 then do; 
    call execute("data &Qtrlib.."||strip(memname)||";"); 
    call execute(" set &Qtrlib.."||strip(memname)||";"); 
    call execute("outsort_1000000=round(1000000*outsort);"); 
    call execute(" run;"); 
end; 
run; 


*---------------------------------------------------* 
| Create the Interpolated Data Sets                 | 
*---------------------------------------------------*; 

data _null_; 
   set work.eval_current nobs=numobs; 
   call symputx("memname_"||strip(_n_),substr(memname,1,30)); 
   if _n_=1 then call symputx("totrecsC",numobs); 
run; 


%let VLIST2=&Interpolate1 &Interpolate2;   **** do not use 2 variables with same first letter for variable in interpolation list - the first letter becomes the output ds prefix; 
%let loopnum2=1; 
%let loopvar2=%scan(&vlist2,1); 
%do %while(&loopvar2 ne ); 

%let prefix=%substr(&loopvar2,1,1); 
%put prefix: &prefix; 

		%do j=1 %to &totrecsC;      
		   %let inname=&qtrlib..&&memname_&j; 
		   %let outname=&qtrlib..&prefix._&&memname_&j; 
		   %put inname: &inname; 
		   %put outname: &outname; 
		   %put loopvar2: &loopvar2; 
           
            %delit(ds=&outname);

            *** create a new dataset with metadata only and 0 observations so base data exists for subsequent proc append;
			data &outname;
			 if 0 then set &inname; stop;
			run;

		    %getInterpResults_V2(in = &inname, 
		                      	 out= &outname, 
		                      	 indep = &loopvar2., 
		                      	 depvars = &depvars, 
		                      	 start = &&start&loopnum2, 
		                      	 incr =  &&incr&loopnum2, 
		                      	 stop =  &&stop&loopnum2); 

		    *-------------------------------------------------------------------------------------------------------------------------* 
		    | Need to add a hack to Interpolated Data Sets - Some values are null when the interpolated value is exactly on increment | 
		    *-------------------------------------------------------------------------------------------------------------------------*; 

		    data &outname; 
		        set &outname; 

		         if _n_=1 then do; 
		         declare hash x(dataset: "&inname"); 
		            x.defineKey("&loopvar2"); 
		            x.defineData('srate','totFrdDol','totLgtDol','cutoff','numFP','dolFP','detFrdDol','detFrdAcct','OSF','Outsort','outsort_1000000'); 
		            x.definedone(); 
		         end; 

		         rc=(x.find()>0); 
				 holder=1;
		         run; 
             ***** add a row of zeros to each output file but only if zeros do not exist on interpolated data;
			 %let min_metric=0;

			 proc sql noprint;
			  select min(&loopvar2) 
              into :min_metric
			  from &outname;

			  %put The Metric is: &loopvar2;
			  %put The smallest interpolated value is: &min_metric;

			  %if &min_metric GT 0 %then %do;

				    %delit(ds=zeros);
				
	            	data zeros; 
					    x=1; 
	                    set &outname  point=x; 
						  array one{*} _numeric_; 
						  do j=1 to dim(one); 
						    one{j}=0; 
						  end; rc=1; 
					    output; stop; drop j; 
					 run; 
	                
					data &outname; 
					 set zeros 
	                     &outname; 
					run; 
					%delit(ds=zeros);
		      %end; *** end of adding zeros to interpolated metric;
            *** dedupe output file - dupes can still be generated if a the largest cutoff values have more than 1 row with zero values of the metric ***;
			  proc sort data=&outname nodupkey;
			    by &loopvar2 descending cutoff;
              run;
			  proc sort data=&outname nodupkey;
			    by &loopvar2;
              run;

		%end; *** end of interpolated data sets; 


*-------------------------------------------------------------------------------------* 
| Add label and Dates to Concatenated - This is main data set for graphics comparison | 
*-------------------------------------------------------------------------------------*; 
%if %symexist(numobs) %then %symdel numobs ; 
%numobs(&loopvar._current); 
options spool; 

    data _null_; 
    do k=1 to 2; 
     do j=1 to &numobs; 
     set &loopvar._current point=j; 
        format date1 date2; 

        if k=1 then do; 
            if j=1 then do; 
              call execute("data &prefix._&loopvar._driver/view=&prefix._&loopvar._driver;"); 
              call execute(" set "); 
            end; 

              call execute("&qtrlib..&prefix._"||strip(memname)||"(in=ds_"||strip(j)||")"); 

              if j=&numobs then call execute(";"); 
        end; 
        else do;  ** loop through the ds a second time; 
             call execute("if ds_"||strip(j)||" then do;"); 
                 call execute("Date1="||date1||";"); 
                 call execute("Date2="||date2||";"); 
                 call execute("Label='"||strip(label)||"';"); 
             call execute("end;"); 
             if j=&numobs then do; 
                call execute("format date1 date2 mmddyy10.;"); 
                call execute("run;"); 
             end; 
        end; 
    end; ** end loop j **; 
    end; ** end loop k **; 
    stop; 
    run; 

	%let loopnum2=%eval(&loopnum2+1); 
%let loopvar2=%scan(&vlist2,&loopnum2); 
%end; 


%end; *** end of processing for EVAL datasets; 



  %let loopnum=%eval(&loopnum+1); 
%end; *** end of processing for STAT and EVAL datasets; 

*** put current ds/label into Macro Variables ***;
data _null_;
 x=1;
 set eval_current point=x;
 call symputx("current_ds","C_"||substr(memname,1,30),'G');
 call symputx("current_ds_raw",memname,'G');
 call symputx("current_lab",label,'G');
 stop;
 run;

%put current_ds: &current_ds;
%put current_ds_raw: &current_ds_raw;
%put current_lab: &current_lab;

%mend prep_driver_ds_interpolate_ds;
